/* Unit testing for outcomes
(C) 2013-2024 Niall Douglas <http://www.nedproductions.biz/> (8 commits)


Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
*/

#include <boost/outcome/result.hpp>
#include <boost/outcome/try.hpp>
#include <boost/test/unit_test.hpp>
#include <boost/test/unit_test_monitor.hpp>

BOOST_OUTCOME_AUTO_TEST_CASE(issues_0059_test, "result<NonMovable> supported?")
{
#if (defined(_MSC_VER) && !defined(__clang__)) || __cplusplus >= 201700
  using namespace BOOST_OUTCOME_V2_NAMESPACE;
  struct udt
  {
    const char *_v{nullptr};
    udt() = delete;
    constexpr udt(const char *v) noexcept : _v(v) {}  // NOLINT
    udt(udt &&o) = delete;
    udt(const udt &) = delete;
    udt &operator=(udt &&o) = delete;
    udt &operator=(const udt &) = delete;
    ~udt() = default;
    constexpr const char *operator*() const noexcept { return _v; }
  };
  const char *niall = "niall";
  auto f = [niall]() -> result<void> {
    auto g = [niall]() -> result<udt> { return {niall}; };
    BOOST_OUTCOME_TRY((auto &&, v), g());  // this must never copy nor move
    BOOST_CHECK(*v == niall);
    return success();
  };
  (void) f();
#endif
}
